import 'reflect-metadata'
import 'dotenv/config'
import jwt from 'jsonwebtoken'
import { ApolloServer } from 'apollo-server-express'
import { ApolloServerPluginDrainHttpServer } from 'apollo-server-core'
import http from 'http'
import app from '../src/server/initApp'
import '../src/server/initDatabaseClient'
import { buildSchemaSync } from 'type-graphql'
import * as resolversObj from '../src/resolvers'
import { objectToArray } from '../src/utils'

// schema auth checker
function authChecker(root: { context: { user?: { uid: string } } }) {
  return root.context.user?.uid ? true : false
}

// build schema
const resolvers = objectToArray<Function>(resolversObj) as [Function, ...Function[]]
const schema = buildSchemaSync({ resolvers, authChecker })

// get auth headers from request context and verify token
async function context({ req }: { req: { headers: { authorization?: string } } }) {
  if (req.headers.authorization && req.headers.authorization.indexOf('Bearer ') === 0) {
    try {
      const accessToken = req.headers.authorization.slice(7)
      const user = jwt.verify(accessToken, process.env.JWT_AUTH_SECRET)
      return { user }
    } catch (e) {
      console.error('Auth error:', e)
    }
  }
}

// create HTTP server
const httpServer = http.createServer(app)

// init apollo server
const server = new ApolloServer({
  schema,
  context,
  introspection: true,
  plugins: [ApolloServerPluginDrainHttpServer({ httpServer })],
})

// link apollo server with express
server.start().then(() => {
  server.applyMiddleware({ app, path: '/api' })
})

export default httpServer

